@@ -12,14 +12,13 @@ import socket
 import datetime
 import json
 import time
+import typing
+
 from collections import OrderedDict
 from pathlib import Path
 from queue import Queue, Empty
-# noinspection PyPackageRequirements
-from kafka import KafkaProducer
-from elasticsearch import Elasticsearch, helpers
+
 from threading import Lock, Thread
-import pymongo
 import requests
 import logging
 from logging import handlers
@@ -30,373 +29,13 @@ from logging.handlers import WatchedFileHandler
 from nb_filelock import FileLock
 from pythonjsonlogger.jsonlogger import JsonFormatter
 from nb_log import nb_log_config_default
-from nb_log import nb_print
+from nb_log.monkey_print import nb_print
+from nb_log.rotate_file_writter import OsFileWritter
 
 very_nb_print = nb_print
 os_name = os.name
 
-
-class MongoHandler(logging.Handler):
-    """
-    一个mongodb的log handler,支持日志按loggername创建不同的集合写入mongodb中
-    """
-
-    # msg_pattern = re.compile('(\d+-\d+-\d+ \d+:\d+:\d+) - (\S*?) - (\S*?) - (\d+) - (\S*?) - ([\s\S]*)')
-
-    def __init__(self, mongo_url, mongo_database='logs'):
-        """
-        :param mongo_url:  mongo连接
-        :param mongo_database: 保存日志的数据库，默认使用logs数据库
-        """
-        logging.Handler.__init__(self)
-        mongo_client = pymongo.MongoClient(mongo_url)
-        self.mongo_db = mongo_client.get_database(mongo_database)
-
-    def emit(self, record):
-        # noinspection PyBroadException, PyPep8
-        try:
-            """以下使用解析日志模板的方式提取出字段"""
-            # msg = self.format(record)
-            # logging.LogRecord
-            # msg_match = self.msg_pattern.search(msg)
-            # log_info_dict = {'time': msg_match.group(1),
-            #                  'name': msg_match.group(2),
-            #                  'file_name': msg_match.group(3),
-            #                  'line_no': msg_match.group(4),
-            #                  'log_level': msg_match.group(5),
-            #                  'detail_msg': msg_match.group(6),
-            #                  }
-            level_str = None
-            if record.levelno == 10:
-                level_str = 'DEBUG'
-            elif record.levelno == 20:
-                level_str = 'INFO'
-            elif record.levelno == 30:
-                level_str = 'WARNING'
-            elif record.levelno == 40:
-                level_str = 'ERROR'
-            elif record.levelno == 50:
-                level_str = 'CRITICAL'
-            log_info_dict = OrderedDict()
-            log_info_dict['time'] = time.strftime('%Y-%m-%d %H:%M:%S')
-            log_info_dict['name'] = record.name
-            log_info_dict['file_path'] = record.pathname
-            log_info_dict['file_name'] = record.filename
-            log_info_dict['func_name'] = record.funcName
-            log_info_dict['line_no'] = record.lineno
-            log_info_dict['log_level'] = level_str
-            log_info_dict['detail_msg'] = record.msg
-            col = self.mongo_db.get_collection(record.name)
-            col.insert_one(log_info_dict)
-        except (KeyboardInterrupt, SystemExit):
-            raise
-        except Exception:
-            self.handleError(record)
-
-
-class KafkaHandler(logging.Handler):
-    """
-    日志批量写入kafka中。
-    """
-    ES_INTERVAL_SECONDS = 0.5
-
-    host_name = socket.gethostname()
-    host_process = f'{host_name} -- {os.getpid()}'
-
-    script_name = sys.argv[0].split('/')[-1]
-
-    task_queue = Queue()
-    last_es_op_time = time.time()
-    has_start_do_bulk_op = False
-    has_start_check_size_and_clear = False
-
-    kafka_producer = None
-    es_index_prefix = 'pylog-'
-
-    def __init__(self, bootstrap_servers, **configs):
-        """
-        :param elastic_hosts:  es的ip地址，数组类型
-        :param elastic_port：  es端口
-        :param index_prefix: index名字前缀。
-        """
-        logging.Handler.__init__(self)
-        if not self.__class__.kafka_producer:
-            very_nb_print('实例化kafka producer')
-            self.__class__.kafka_producer = KafkaProducer(bootstrap_servers=bootstrap_servers, **configs)
-
-        t = Thread(target=self._do_bulk_op)
-        t.setDaemon(True)
-        t.start()
-
-    @classmethod
-    def __add_task_to_bulk(cls, task):
-        cls.task_queue.put(task)
-
-    # noinspection PyUnresolvedReferences
-    @classmethod
-    def __clear_bulk_task(cls):
-        cls.task_queue.queue.clear()
-
-    @classmethod
-    def _check_size_and_clear(cls):
-        """
-        如果是外网传输日志到测试环境风险很大，测试环境网络经常打满，传输不了会造成日志队列堆积，会造成内存泄漏，所以需要清理。
-        :return:
-        """
-        if cls.has_start_check_size_and_clear:
-            return
-        cls.has_start_check_size_and_clear = True
-
-        def __check_size_and_clear():
-            while 1:
-                size = cls.task_queue.qsize()
-                if size > 1000:
-                    very_nb_print(f'kafka防止意外日志积累太多了,达到 {size} 个，为防止内存泄漏，清除队列')
-                    cls.__clear_bulk_task()
-                time.sleep(0.1)
-
-        t = Thread(target=__check_size_and_clear)
-        t.setDaemon(True)
-        t.start()
-
-    @classmethod
-    def _do_bulk_op(cls):
-        if cls.has_start_do_bulk_op:
-            return
-
-        cls.has_start_do_bulk_op = True
-        # very_nb_print(cls.kafka_producer)
-        while 1:
-            try:
-                # noinspection PyUnresolvedReferences
-                tasks = list(cls.task_queue.queue)
-                cls.__clear_bulk_task()
-                for task in tasks:
-                    topic = (cls.es_index_prefix + task['name']).replace('.', '').replace('_', '').replace('-', '')
-                    # very_nb_print(topic)
-                    cls.kafka_producer.send(topic, json.dumps(task).encode())
-                cls.last_es_op_time = time.time()
-            except Exception as e:
-                very_nb_print(e)
-            finally:
-                time.sleep(cls.ES_INTERVAL_SECONDS)
-
-    def emit(self, record):
-        # noinspection PyBroadException, PyPep8
-        try:
-            level_str = None
-            if record.levelno == 10:
-                level_str = 'DEBUG'
-            elif record.levelno == 20:
-                level_str = 'INFO'
-            elif record.levelno == 30:
-                level_str = 'WARNING'
-            elif record.levelno == 40:
-                level_str = 'ERROR'
-            elif record.levelno == 50:
-                level_str = 'CRITICAL'
-            log_info_dict = OrderedDict()
-            log_info_dict['@timestamp'] = datetime.datetime.utcfromtimestamp(record.created).isoformat()
-            log_info_dict['time'] = time.strftime('%Y-%m-%d %H:%M:%S')
-            log_info_dict['name'] = record.name
-            log_info_dict['host'] = self.host_name
-            log_info_dict['host_process'] = self.host_process
-            # log_info_dict['file_path'] = record.pathname
-            log_info_dict['file_name'] = record.filename
-            log_info_dict['func_name'] = record.funcName
-            # log_info_dict['line_no'] = record.lineno
-            log_info_dict['log_place'] = f'{record.pathname}:{record.lineno}'
-            log_info_dict['log_level'] = level_str
-            log_info_dict['msg'] = str(record.msg)
-            log_info_dict['script'] = self.script_name
-            log_info_dict['es_index'] = f'{self.es_index_prefix}{record.name.lower()}'
-            self.__add_task_to_bulk(log_info_dict)
-
-        except (KeyboardInterrupt, SystemExit):
-            raise
-        except Exception:
-            self.handleError(record)
-
-
-class ElasticHandler000(logging.Handler):
-    """
-    日志批量写入es中。
-    """
-    ES_INTERVAL_SECONDS = 2
-    host_name = socket.gethostname()
-
-    def __init__(self, elastic_hosts: list, elastic_port, index_prefix='pylog-'):
-        """
-        :param elastic_hosts:  es的ip地址，数组类型
-        :param elastic_port：  es端口
-        :param index_prefix: index名字前缀。
-        """
-        logging.Handler.__init__(self)
-        self._es_client = Elasticsearch(elastic_hosts, port=elastic_port)
-        self._index_prefix = index_prefix
-        self._task_list = []
-        self._task_queue = Queue()
-        self._last_es_op_time = time.time()
-        t = Thread(target=self._do_bulk_op)
-        t.setDaemon(True)
-        t.start()
-
-    def __add_task_to_bulk(self, task):
-        self._task_queue.put(task)
-
-    def __clear_bulk_task(self):
-        # noinspection PyUnresolvedReferences
-        self._task_queue.queue.clear()
-
-    def _do_bulk_op(self):
-        while 1:
-            try:
-                if self._task_queue.qsize() > 10000:
-                    very_nb_print('防止意外日志积累太多了，不插入es了。')
-                    self.__clear_bulk_task()
-                    return
-                # noinspection PyUnresolvedReferences
-                tasks = list(self._task_queue.queue)
-                self.__clear_bulk_task()
-                helpers.bulk(self._es_client, tasks)
-
-                self._last_es_op_time = time.time()
-            except Exception as e:
-                very_nb_print(e)
-            finally:
-                time.sleep(1)
-
-    def emit(self, record):
-        # noinspection PyBroadException, PyPep8
-        try:
-            level_str = None
-            if record.levelno == 10:
-                level_str = 'DEBUG'
-            elif record.levelno == 20:
-                level_str = 'INFO'
-            elif record.levelno == 30:
-                level_str = 'WARNING'
-            elif record.levelno == 40:
-                level_str = 'ERROR'
-            elif record.levelno == 50:
-                level_str = 'CRITICAL'
-            log_info_dict = OrderedDict()
-            log_info_dict['@timestamp'] = datetime.datetime.utcfromtimestamp(record.created).isoformat()
-            log_info_dict['time'] = time.strftime('%Y-%m-%d %H:%M:%S')
-            log_info_dict['name'] = record.name
-            log_info_dict['host'] = self.host_name
-            log_info_dict['file_path'] = record.pathname
-            log_info_dict['file_name'] = record.filename
-            log_info_dict['func_name'] = record.funcName
-            log_info_dict['line_no'] = record.lineno
-            log_info_dict['log_level'] = level_str
-            log_info_dict['msg'] = str(record.msg)
-            self.__add_task_to_bulk({
-                "_index": f'{self._index_prefix}{record.name.lower()}',
-                "_type": f'{self._index_prefix}{record.name.lower()}',
-                "_source": log_info_dict
-            })
-
-        except (KeyboardInterrupt, SystemExit):
-            raise
-        except Exception:
-            self.handleError(record)
-
-
-# noinspection PyUnresolvedReferences
-class ElasticHandler(logging.Handler):
-    """
-    日志批量写入es中。
-    """
-    ES_INTERVAL_SECONDS = 0.5
-
-    host_name = socket.gethostname()
-    host_process = f'{host_name} -- {os.getpid()}'
-
-    script_name = sys.argv[0]
-
-    task_queue = Queue()
-    last_es_op_time = time.time()
-    has_start_do_bulk_op = False
-
-    def __init__(self, elastic_hosts: list, elastic_port, index_prefix='pylog-'):
-        """
-        :param elastic_hosts:  es的ip地址，数组类型
-        :param elastic_port：  es端口
-        :param index_prefix: index名字前缀。
-        """
-        logging.Handler.__init__(self)
-        self._es_client = Elasticsearch(elastic_hosts, port=elastic_port)
-        self._index_prefix = index_prefix
-        t = Thread(target=self._do_bulk_op)
-        t.setDaemon(True)
-        t.start()
-
-    @classmethod
-    def __add_task_to_bulk(cls, task):
-        cls.task_queue.put(task)
-
-    # noinspection PyUnresolvedReferences
-    @classmethod
-    def __clear_bulk_task(cls):
-        cls.task_queue.queue.clear()
-
-    def _do_bulk_op(self):
-        if self.__class__.has_start_do_bulk_op:
-            return
-        self.__class__.has_start_do_bulk_op = True
-        while 1:
-            try:
-                if self.__class__.task_queue.qsize() > 10000:
-                    very_nb_print('防止意外日志积累太多了，不插入es了。')
-                    self.__clear_bulk_task()
-                    return
-                tasks = list(self.__class__.task_queue.queue)
-                self.__clear_bulk_task()
-                helpers.bulk(self._es_client, tasks)
-                self.__class__.last_es_op_time = time.time()
-            except Exception as e:
-                very_nb_print(e)
-            finally:
-                time.sleep(self.ES_INTERVAL_SECONDS)
-
-    def emit(self, record):
-        # noinspection PyBroadException, PyPep8
-        try:
-            level_str = None
-            if record.levelno == 10:
-                level_str = 'DEBUG'
-            elif record.levelno == 20:
-                level_str = 'INFO'
-            elif record.levelno == 30:
-                level_str = 'WARNING'
-            elif record.levelno == 40:
-                level_str = 'ERROR'
-            elif record.levelno == 50:
-                level_str = 'CRITICAL'
-            log_info_dict = OrderedDict()
-            log_info_dict['@timestamp'] = datetime.datetime.utcfromtimestamp(record.created).isoformat()
-            log_info_dict['time'] = time.strftime('%Y-%m-%d %H:%M:%S')
-            log_info_dict['name'] = record.name
-            log_info_dict['host'] = self.host_name
-            log_info_dict['host_process'] = self.host_process
-            log_info_dict['file_path'] = record.pathname
-            log_info_dict['file_name'] = record.filename
-            log_info_dict['func_name'] = record.funcName
-            log_info_dict['line_no'] = record.lineno
-            log_info_dict['log_level'] = level_str
-            log_info_dict['msg'] = str(record.msg)
-            log_info_dict['script'] = self.script_name
-            self.__add_task_to_bulk({
-                "_index": f'{self._index_prefix}{record.name.lower()}',
-                "_type": f'{self._index_prefix}{record.name.lower()}',
-                "_source": log_info_dict
-            })
-
-        except (KeyboardInterrupt, SystemExit):
-            raise
-        except Exception:
-            self.handleError(record)
+host_name = socket.gethostname()
 
 
 # noinspection PyPep8Naming
@@ -595,6 +234,7 @@ class ColorHandler(logging.Handler):
             stream = self.stream
             # print(assist_msg)
             # print(effective_information_msg)
+            # record.raw_no_color_msg = record.msg
             if nb_log_config_default.DISPLAY_BACKGROUD_COLOR_IN_CONSOLE:
                 msg_color = self.__build_color_msg_with_backgroud_color(record.levelno, copy.copy(record))
             else:
@@ -612,11 +252,11 @@ class ColorHandler(logging.Handler):
     @staticmethod
     def __spilt_msg(log_level, msg: str):
         split_text = '- 级别 -'
-        if log_level == 10:
+        if log_level == logging.DEBUG:
             split_text = '- DEBUG -'
-        elif log_level == 20:
+        elif log_level == logging.INFO:
             split_text = '- INFO -'
-        elif log_level == 30:
+        elif log_level == logging.WARNING:
             split_text = '- WARNING -'
         elif log_level == 40:
             split_text = '- ERROR -'
@@ -649,7 +289,7 @@ class ConcurrentRotatingFileHandlerWithBufferInitiativeWindwos(ConcurrentRotatin
             for hr in cls.file_handler_list:
                 # very_nb_print(hr.buffer_msgs_queue.qsize())
                 hr.rollover_and_do_write()
-            time.sleep(1)
+            time.sleep(0.1)
 
     @classmethod
     def start_emit_all_file_handler(cls):
@@ -658,12 +298,12 @@ class ConcurrentRotatingFileHandlerWithBufferInitiativeWindwos(ConcurrentRotatin
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.buffer_msgs_queue = Queue()
+        self.buffer_msgs_queue = queue.SimpleQueue()
         atexit.register(self._when_exit)  # 如果程序属于立马就能结束的，需要在程序结束前执行这个钩子，防止不到最后一秒的日志没记录到。
         self.file_handler_list.append(self)
         if not self.has_start_emit_all_file_handler:
-            self.start_emit_all_file_handler()
             self.__class__.has_start_emit_all_file_handler = True
+            self.start_emit_all_file_handler()
 
     def _when_exit(self):
         pass
@@ -692,6 +332,8 @@ class ConcurrentRotatingFileHandlerWithBufferInitiativeWindwos(ConcurrentRotatin
             try:
                 msg = self.buffer_msgs_queue.get(block=False)
                 buffer_msgs += msg + '\n'
+                # if len(buffer_msgs) > 1000 * 1000 * 100:
+                #     break
             except Empty:
                 break
         if buffer_msgs:
@@ -722,7 +364,7 @@ class ConcurrentRotatingFileHandlerWithBufferInitiativeLinux00000000(ConcurrentR
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.buffer_msgs_queue = Queue()
-        atexit.register(self._when_exit)  # 如果程序属于立马就能结束的，需要在程序结束前执行这个钩子，防止不到最后一秒的日志没记录到。
+        atexit.register(self._when_exit)  # 如果程序属于立马就能结束的，需要在程序结束前执行这个钩子，防止不到最后一秒的日志没记录到。   有个弊端 at_exit 不能在子进程生效，所以不用这个类
         self.file_handler_list.append(self)
         if os.getpid() not in self.has_start_emit_all_file_handler_process_id_set:
             self.start_emit_all_file_handler()
@@ -883,6 +525,7 @@ class DingTalkHandler(logging.Handler):
                     logging.getLogger(logger_name).handlers.pop(index)
 
 
+# noinspection PyPep8Naming
 class ConcurrentDayRotatingFileHandlerWin(logging.Handler):
     """
     这个多进程按时间切片安全的。
@@ -896,6 +539,7 @@ class ConcurrentDayRotatingFileHandlerWin(logging.Handler):
     @classmethod
     def _emit_all_file_handler(cls):
         while True:
+            # print(os.getpid())
             for hr in cls.file_handler_list:
                 # very_nb_print(hr.buffer_msgs_queue.qsize())
                 # noinspection PyProtectedMember
@@ -908,14 +552,14 @@ class ConcurrentDayRotatingFileHandlerWin(logging.Handler):
         Thread(target=cls._emit_all_file_handler, daemon=True).start()
 
     # noinspection PyMissingConstructor
-    def __init__(self, file_name: str, file_path: str, back_count=10):
+    def __init__(self, file_name: str, file_path: str, back_count=None):
         super().__init__()
         self.file_name = file_name
         self.file_path = file_path
-        self.backupCount = back_count
+        self.backupCount = back_count or nb_log_config_default.LOG_FILE_BACKUP_COUNT
         self.extMatch = re.compile(r"^\d{4}-\d{2}-\d{2}(\.\w+)?$", re.ASCII)
         self.extMatch2 = re.compile(r"^\d{2}-\d{2}-\d{2}(\.\w+)?$", re.ASCII)
-        self._last_delete_time = time.time()
+        self._last_delete_time = 0
 
         self.buffer_msgs_queue = queue.Queue()
         atexit.register(self._write_to_file)  # 如果程序属于立马就能结束的，需要在程序结束前执行这个钩子，防止不到最后一秒的日志没记录到。
@@ -937,13 +581,16 @@ class ConcurrentDayRotatingFileHandlerWin(logging.Handler):
         except Exception:
             self.handleError(record)
 
-    def _write_to_file(self):
+    def _write_to_file(self):  # 也可以重写 close方法，来处理末尾。
         buffer_msgs = ''
         while True:
             # print(self.buffer_msgs_queue.qsize())
             try:
                 msg = self.buffer_msgs_queue.get(block=False)
                 buffer_msgs += msg + '\n'
+                if len(buffer_msgs) > 1000 * 1000 * 10:
+                    pass
+                    break
             except queue.Empty:
                 break
         if buffer_msgs:
@@ -953,8 +600,9 @@ class ConcurrentDayRotatingFileHandlerWin(logging.Handler):
                 new_file_name = self.file_name + '.' + time_str
                 path_obj = Path(self.file_path) / Path(new_file_name)
                 path_obj.touch(exist_ok=True)
-                with path_obj.open(mode='a') as f:
+                with path_obj.open(mode='a', encoding='utf-8') as f:
                     f.write(buffer_msgs)
+                    f.flush()
                 if time.time() - self._last_delete_time > 60:
                     self._find_and_delete_files()
                     self._last_delete_time = time.time()
@@ -988,12 +636,13 @@ class ConcurrentDayRotatingFileHandlerWin(logging.Handler):
             Path(r).unlink()
 
 
+# noinspection PyPep8Naming
 class ConcurrentDayRotatingFileHandlerLinux(logging.Handler):
-    def __init__(self, file_name: str, file_path: str, back_count=10):
+    def __init__(self, file_name: str, file_path: str, back_count=None):
         super().__init__()
         self.file_name = file_name
         self.file_path = file_path
-        self.backupCount = back_count
+        self.backupCount = back_count or nb_log_config_default.LOG_FILE_BACKUP_COUNT
         self.extMatch = re.compile(r"^\d{4}-\d{2}-\d{2}(\.\w+)?$", re.ASCII)
         self.extMatch2 = re.compile(r"^\d{2}-\d{2}-\d{2}(\.\w+)?$", re.ASCII)
         self._last_delete_time = time.time()
@@ -1031,13 +680,23 @@ class ConcurrentDayRotatingFileHandlerLinux(logging.Handler):
         try:
             msg = self.format(record)
             self.fp.write(msg + '\n')
-        except Exception:
+            self.fp.flush()  # 需要flush才能及时写入。重写close可以写入程序结束前的缓冲。
+        except Exception as e:
+            print(e)
             self.handleError(record)
         if time.time() - self._last_delete_time > 60:
             self._get_fp()
             self._find_and_delete_files()
             self._last_delete_time = time.time()
 
+    def close(self):
+        with self._lock:
+            try:
+                self.fp.flush()
+                self.fp.close()
+            except Exception as e:
+                print(e)
+
     def _find_and_delete_files(self):
         """
         这一段命名不规范是复制原来的官方旧代码。
@@ -1067,17 +726,18 @@ class ConcurrentDayRotatingFileHandlerLinux(logging.Handler):
             Path(r).unlink()
 
 
-class ConcurrentSecondRotatingFileHandlerLinux(logging.Handler):
+# noinspection PyPep8Naming
+class _ConcurrentSecondRotatingFileHandlerLinux(logging.Handler):
     """ 按秒切割的多进程安全文件日志，方便测试验证"""
-    def __init__(self, file_name: str, file_path: str, back_count=10):
+
+    def __init__(self, file_name: str, file_path: str, back_count=None):
         super().__init__()
         self.file_name = file_name
         self.file_path = file_path
-        self.backupCount = back_count
+        self.backupCount = back_count or nb_log_config_default.LOG_FILE_BACKUP_COUNT
         self.extMatch = re.compile(r"^\d{4}-\d{2}-\d{2}(\.\w+)?$", re.ASCII)
         self.extMatch2 = re.compile(r"^\d{2}-\d{2}-\d{2}(\.\w+)?$", re.ASCII)
-        self._last_delete_time = time.time()
-
+        self._last_delete_time = 0
 
         time_str = time.strftime('%H-%M-%S')  # 方便测试用的，方便观察。
         new_file_name = self.file_name + '.' + time_str
@@ -1111,16 +771,15 @@ class ConcurrentSecondRotatingFileHandlerLinux(logging.Handler):
         # noinspection PyBroadException
         try:
             msg = self.format(record)
-            self.fp.write(msg + '\n')
             if time.time() - self._last_delete_time > 0.5:
                 self._get_fp()
                 self._find_and_delete_files()
                 self._last_delete_time = time.time()
+            self.fp.write(msg + '\n')
         except Exception as e:
             print(e)
             self.handleError(record)
 
-
     def _find_and_delete_files(self):
         """
         这一段命名不规范是复制原来的官方旧代码。
@@ -1150,13 +809,27 @@ class ConcurrentSecondRotatingFileHandlerLinux(logging.Handler):
             try:
                 Path(r).unlink()
                 print(f'删除成功 {r}')
-            except (PermissionError,FileNotFoundError) as e:
+            except (PermissionError, FileNotFoundError) as e:
                 print(e)
 
 
-
 ConcurrentDayRotatingFileHandler = ConcurrentDayRotatingFileHandlerWin if os_name == 'nt' else ConcurrentDayRotatingFileHandlerLinux
 
+
 # ConcurrentDayRotatingFileHandler = ConcurrentSecondRotatingFileHandlerLinux
 #
 # print(ConcurrentDayRotatingFileHandler)
+
+
+class BothDayAndSizeRotatingFileHandler(logging.Handler):
+    """
+    自己从头开发的按照时间和大小切割
+    """
+
+    def __init__(self, file_name: typing.Optional[str], log_path='/pythonlogs', max_bytes=1000 * 1000 * 1000, back_count=10):
+        super().__init__()
+        self.os_file_writter = OsFileWritter(file_name=file_name, log_path=log_path, max_bytes=max_bytes, back_count=back_count)
+
+    def emit(self, record: logging.LogRecord) -> None:
+        msg = self.format(record)
+        self.os_file_writter.write_2_file(msg + '\n')
