@@ -19,12 +19,24 @@ concurrent_log_handler的ConcurrentRotatingFileHandler解决了logging模块自
 使极限多进程安全切片的文件日志写入性能在win下提高100倍，linux下提高10倍。
 
 """
+import logging
 import multiprocessing
 import typing
 from functools import lru_cache
-from logging import FileHandler
-from nb_log import nb_log_config_default  # noqa
+from logging import FileHandler, _checkLevel  # noqa
+from nb_log import nb_log_config_default, compatible_logger  # noqa
+from nb_log.compatible_logger import CompatibleLogger
 from nb_log.handlers import *
+import deprecated
+
+MANUAL_HANLDER_TYPE = 'manual_hanlder_type'
+HANDLER_TYPE_FILE = 'HANDLER_TYPE_FILE'
+HANDLER_TYPE_ERROR_FILE = 'HANDLER_TYPE_ERROR_FILE'
+HANDLER_TYPE_STREAM = 'HANDLER_TYPE_STREAM'
+
+
+def _get_hanlder_type(handlerx: logging.Handler):
+    return getattr(handlerx, MANUAL_HANLDER_TYPE, None) or type(handlerx)
 
 
 # noinspection DuplicatedCode
@@ -66,9 +78,9 @@ def revision_call_handlers(self, record):  # 对logging标准模块打猴子补
 
     while c:
         for hdlr in c.handlers:
-            hdlr_type = type(hdlr)
-            if hdlr_type == logging.StreamHandler:  # REMIND 因为很多handler都是继承自StreamHandler，包括filehandler，直接判断会逻辑出错。
-                hdlr_type = ColorHandler
+            hdlr_type = _get_hanlder_type(hdlr)
+            # if hdlr_type == logging.StreamHandler:  # REMIND 因为很多handler都是继承自StreamHandler，包括filehandler，直接判断会逻辑出错。
+            #     hdlr_type = ColorHandler
             found = found + 1
             if record.levelno >= hdlr.level:
                 if hdlr_type not in hdlr_type_set:
@@ -104,22 +116,35 @@ def revision_add_handler(self, hdlr):  # 从添加源头阻止同一个logger添
         """
         hdlrx_type_set = set()
         for hdlrx in self.handlers:
-            hdlrx_type = type(hdlrx)
-            if hdlrx_type == logging.StreamHandler:  # REMIND 因为很多handler都是继承自StreamHandler，包括filehandler，直接判断会逻辑出错。
-                hdlrx_type = ColorHandler
+            hdlrx_type = _get_hanlder_type(hdlrx)
+            # if hdlrx_type == logging.StreamHandler:  # REMIND 因为很多handler都是继承自StreamHandler，包括filehandler，直接判断会逻辑出错。
+            #     hdlrx_type = ColorHandler
             hdlrx_type_set.add(hdlrx_type)
 
-        hdlr_type = type(hdlr)
-        if hdlr_type == logging.StreamHandler:
-            hdlr_type = ColorHandler
+        hdlr_type = _get_hanlder_type(hdlr)
+        # if hdlr_type == logging.StreamHandler:
+        #     hdlr_type = ColorHandler
         if hdlr_type not in hdlrx_type_set:
             self.handlers.append(hdlr)
     finally:
         logging._releaseLock()  # noqa
 
 
+def revision_setLevel(self, level):
+    """
+    Set the logging level of this logger.  level must be an int or a str.
+    """
+    level2 = LogManager.preset_name__level_map.get(self.name, level)
+    if level2 != level:
+        very_nb_print(f'日志命名空间 {self.name} 已经锁定了为了 {level2} 级别 ,后续不可以更改为 {level} 级别')
+    self.level = _checkLevel(level2)
+    if sys.version_info.minor >= 7:  # python3.6 没有 _clear_cache 方法
+        self.manager._clear_cache()
+
+
 logging.Logger.callHandlers = revision_call_handlers  # 打猴子补丁。
 logging.Logger.addHandler = revision_add_handler  # 打猴子补丁。
+logging.Logger.setLevel = revision_setLevel  # 打猴子补丁。
 
 
 # noinspection PyShadowingBuiltins
@@ -180,6 +205,19 @@ class MailHandlerConfig(DataClassBase):
     mail_time_interval = 60
 
 
+def get_all_logging_name():
+    logger_names = logging.Logger.manager.loggerDict.keys()
+    return logger_names
+
+
+LOG_LEVEL_LIST = [logging.DEBUG, logging.INFO, logging.WARNING, logging.ERROR, logging.CRITICAL]  # 就是 10 20 30 40 50
+
+
+def check_log_level(log_level: int):
+    if log_level not in LOG_LEVEL_LIST:
+        raise ValueError(f'你设置的日志级别不正确,你设置的级别是 {log_level} ，日志级别必须是 {LOG_LEVEL_LIST} 其中之一')
+
+
 # noinspection PyMissingOrEmptyDocstring,PyPep8
 class LogManager(object):
     """
@@ -189,31 +227,50 @@ class LogManager(object):
     logger_list = []
     preset_name__level_map = dict()
 
-    def __init__(self, logger_name: typing.Union[str, None] = 'nb_log_default_namespace'):
+    @staticmethod
+    def get_all_logging_name():
+        return get_all_logging_name()
+
+    def __init__(self, logger_name: typing.Union[str, None] = 'nb_log_default_namespace', logger_cls=logging.Logger):
         """
         :param logger_name: 日志名称，当为None时候创建root命名空间的日志，一般情况下千万不要传None，除非你确定需要这么做和是在做什么.这个命名空间是双刃剑
         """
-        if logger_name in (None, '', 'root') and multiprocessing.process.current_process().name == 'MainProcess':
-            very_nb_print('logger_name 设置为None和root和空字符串都是一个意义，在操作根日志命名空间，任何其他日志的行为将会发生变化，'
+        if logger_name in (None, '',) and multiprocessing.process.current_process().name == 'MainProcess':
+            very_nb_print('logger_name 设置为None和空字符串都是一个意义，在操作根日志命名空间，任何其他日志的行为将会发生变化，'
                           '一定要弄清楚原生logging包的日志name的意思。这个命名空间是双刃剑')
         self._logger_name = logger_name
-        self.logger = logging.getLogger(logger_name)
+        if logger_cls == logging.Logger:
+            self.logger = logging.getLogger(logger_name)
+        else:
+            self.logger = logger_cls(logger_name)
+
+    @staticmethod
+    def generate_error_file_name(log_filename: str):
+        if log_filename is None:
+            return None
+        arr = log_filename.split('.')
+        part1 = '.'.join(arr[:-1])
+        part2 = arr[-1]
+        return f'{part1}_error.{part2}'
 
     def preset_log_level(self, log_level_int=20):
         """
         提前设置锁定日志级别，当之后再设置该命名空间日志的级别的时候，按照提前预设的级别，无视之后设定的级别。
         主要是针对动态初始化的日志，在生成日志之后再去设置日志级别不方便。
-        :param log_level_int:
+        :param log_level_int:logging.DEBUG LOGGING.INFO 等
         :return:
         """
+        check_log_level(log_level_int)
         self.preset_name__level_map[self._logger_name] = log_level_int
 
     # 加*是为了强制在调用此方法时候使用关键字传参，如果以位置传参强制报错，因为此方法后面的参数中间可能以后随时会增加更多参数，造成之前的使用位置传参的代码参数意义不匹配。
     # noinspection PyAttributeOutsideInit
     def get_logger_and_add_handlers(self, log_level_int: int = None, *, is_add_stream_handler=True,
+                                    is_use_loguru_stream_handler:bool=None,
                                     do_not_use_color_handler=None, log_path=None,
                                     log_filename=None, log_file_size: int = None,
                                     log_file_handler_type: int = None,
+                                    error_log_filename=None,
                                     mongo_url=None, is_add_elastic_handler=False, is_add_kafka_handler=False,
                                     ding_talk_token=None, ding_talk_time_interval=60,
                                     mail_handler_config: MailHandlerConfig = MailHandlerConfig(),
@@ -222,17 +279,21 @@ class LogManager(object):
         """
        :param log_level_int: 日志输出级别，设置为 1 2 3 4 5，分别对应原生logging.DEBUG(10)，logging.INFO(20)，logging.WARNING(30)，logging.ERROR(40),logging.CRITICAL(50)级别，现在可以直接用10 20 30 40 50了，兼容了。
        :param is_add_stream_handler: 是否打印日志到控制台
+       :param is_use_loguru_stream_handler  是否使用 loguru的控制台打印，如果为None，使用 nb_log_config.py的DEFAULUT_IS_USE_LOGURU_STREAM_HANDLER 值。
        :param do_not_use_color_handler :是否禁止使用color彩色日志
        :param log_path: 设置存放日志的文件夹路径,如果不设置，则取nb_log_config.LOG_PATH，如果配置中也没指定则自动在代码所在磁盘的根目录创建/pythonlogs文件夹，
               非windwos下要注意账号权限问题(如果python没权限在根目录建/pythonlogs，则需要手动先创建好)
-       :param log_filename: 日志的名字，仅当log_path和log_filename都不为None时候才写入到日志文件。
+       :param log_filename: 日志文件名字，仅当log_path和log_filename都不为None时候才写入到日志文件。
+       :param error_log_filename :错误日志文件名字，如果文件名不为None，那么error级别以上日志自动写入到这个错误文件。
        :param log_file_size :日志大小，单位M，默认100M
-       :param log_file_handler_type :这个值可以设置为1 2 3 4 四种值，1为使用多进程安全按日志文件大小切割的文件日志
+       :param log_file_handler_type :这个值可以设置为1 2 3 4 5 6 7，1为使用多进程安全按日志文件大小切割的文件日志
               2为多进程安全按天自动切割的文件日志，同一个文件，每天生成一个日志
               3为不自动切割的单个文件的日志(不切割文件就不会出现所谓进程安不安全的问题)
               4为 WatchedFileHandler，这个是需要在linux下才能使用，需要借助lograte外力进行日志文件的切割，多进程安全。
               5 为第三方的concurrent_log_handler.ConcurrentRotatingFileHandler按日志文件大小切割的文件日志，
                 这个是采用了文件锁，多进程安全切割，文件锁在linux上使用fcntl性能还行，win上使用win32con性能非常惨。按大小切割建议不要选第5个个filehandler而是选择第1个。
+              6 为作者发明的高性能多进程安全，同时按大小和时间切割的文件日志handler
+              7 为 loguru的 文件日志记录器
        :param mongo_url : mongodb的连接，为None时候不添加mongohandler
        :param is_add_elastic_handler: 是否记录到es中。
        :param is_add_kafka_handler: 日志是否发布到kafka。
@@ -264,19 +325,23 @@ class LogManager(object):
         if formatter_template is None:
             formatter_template = nb_log_config_default.FORMATTER_KIND
 
-        self._logger_level = log_level_int * 10 if log_level_int < 10 else log_level_int
-        if self._logger_name in self.preset_name__level_map:
-            # print(self.preset_name__level_map)
-            self._logger_level2 = (self.preset_name__level_map[self._logger_name])
-        else:
-            self._logger_level2 = self._logger_level
+        check_log_level(log_level_int)
+        self._logger_level = log_level_int
+        # if self._logger_name in self.preset_name__level_map:
+        #     # print(self.preset_name__level_map)
+        #     self._logger_level = (self.preset_name__level_map[self._logger_name])
+        # else:
+        #     self._logger_level = self._logger_level
         self._is_add_stream_handler = is_add_stream_handler
         self._do_not_use_color_handler = do_not_use_color_handler
         self._log_path = log_path
         self._log_filename = log_filename
+        if error_log_filename is None and nb_log_config_default.AUTO_WRITE_ERROR_LEVEL_TO_SEPARATE_FILE:
+            error_log_filename = self.generate_error_file_name(log_filename)
+        self._error_log_filename = error_log_filename
         self._log_file_size = log_file_size
-        if log_file_handler_type not in (None, 1, 2, 3, 4, 5):
-            raise ValueError("log_file_handler_type的值必须设置为 1 2 3 4这四个数字")
+        if log_file_handler_type not in (None, 1, 2, 3, 4, 5, 6,7):
+            raise ValueError("log_file_handler_type的值必须设置为 1 2 3 4 5 6 7这几个数字")
         self._log_file_handler_type = log_file_handler_type or nb_log_config_default.LOG_FILE_HANDLER_TYPE
         self._mongo_url = mongo_url
         self._is_add_elastic_handler = is_add_elastic_handler
@@ -285,6 +350,9 @@ class LogManager(object):
         self._ding_talk_time_interval = ding_talk_time_interval
         self._mail_handler_config = mail_handler_config
         self._is_add_mail_handler = is_add_mail_handler
+        self._is_use_loguru_stream_handler = nb_log_config_default.DEFAULUT_IS_USE_LOGURU_STREAM_HANDLER  if is_use_loguru_stream_handler is None\
+            else is_use_loguru_stream_handler
+
 
         if isinstance(formatter_template, int):
             self._formatter = nb_log_config_default.FORMATTER_DICT[formatter_template]
@@ -293,7 +361,7 @@ class LogManager(object):
         else:
             raise ValueError('设置的 formatter_template 不正确')
 
-        self.logger.setLevel(self._logger_level2)
+        self.logger.setLevel(self._logger_level)
         self.__add_handlers()
         # self.logger_name_list.append(self._logger_name)
         # self.logger_list.append(self.logger)
@@ -312,19 +380,20 @@ class LogManager(object):
         #     self.logger.removeHandler(hd)
         self.logger.handlers = []
 
-    def remove_handler_by_handler_class(self, handler_class: type):
+    def remove_handler_by_handler_class(self, handler_class: typing.Union[type,str]):
         """
         去掉指定类型的handler
         :param handler_class:logging.StreamHandler,ColorHandler,MongoHandler,ConcurrentRotatingFileHandler,MongoHandler,CompatibleSMTPSSLHandler的一种
         :return:
         """
-        if handler_class not in (
-            logging.StreamHandler, ColorHandler, MongoHandler, ConcurrentRotatingFileHandler, MongoHandler,
-            CompatibleSMTPSSLHandler, ElasticHandler, DingTalkHandler, KafkaHandler):
-            raise TypeError('设置的handler类型不正确')
+        # if handler_class not in (
+        #     logging.StreamHandler, ColorHandler, MongoHandler, ConcurrentRotatingFileHandler, MongoHandler,
+        #     CompatibleSMTPSSLHandler, ElasticHandler, DingTalkHandler, KafkaHandler):
+        #     raise TypeError('设置的handler类型不正确')
         all_handlers = copy.copy(self.logger.handlers)
         for handler in all_handlers:
-            if isinstance(handler, handler_class):
+            # if isinstance(handler, handler_class):
+            if _get_hanlder_type(handler) == handler_class:
                 self.logger.removeHandler(handler)  # noqa
 
     def __add_a_hanlder(self, handlerx: logging.Handler):
@@ -332,33 +401,24 @@ class LogManager(object):
         handlerx.setFormatter(self._formatter)
         self.logger.addHandler(handlerx)
 
-    def _judge_logger_has_handler_type(self, handler_type: type):
+    # def _judge_logger_has_handler_type(self, handler_type: typing.Union[type,str]):
+    #     for hr in self.logger.handlers:
+    #         if isinstance(hr, handler_type):
+    #             return True
+
+    def _judge_logger_has_not_handler_type(self, handler_type: typing.Union[type,str]):
         for hr in self.logger.handlers:
-            if isinstance(hr, handler_type):
-                return True
-
-    def __add_handlers(self):
-        pass
-
-        # REMIND 添加控制台日志
-        if not (self._judge_logger_has_handler_type(ColorHandler) or self._judge_logger_has_handler_type(
-            logging.StreamHandler)) and self._is_add_stream_handler:
-            handler = ColorHandler() if not self._do_not_use_color_handler else logging.StreamHandler()  # 不使用streamhandler，使用自定义的彩色日志
-            # handler = logging.StreamHandler()
-            handler.setLevel(self._logger_level2)
-            self.__add_a_hanlder(handler)
+            if _get_hanlder_type(hr)==handler_type:
+                return False
+        return True
 
+    def __add_file_hanlder(self, log_filename, is_error_level_file_handler):
         # REMIND 添加多进程安全切片的文件日志
-        if not (self._judge_logger_has_handler_type(ConcurrentRotatingFileHandler) or
-                self._judge_logger_has_handler_type(ConcurrentRotatingFileHandlerWithBufferInitiativeWindwos) or
-                self._judge_logger_has_handler_type(ConcurrentRotatingFileHandlerWithBufferInitiativeLinux) or
-                self._judge_logger_has_handler_type(ConcurrentDayRotatingFileHandler) or
-                self._judge_logger_has_handler_type(FileHandler) or
-                self._judge_logger_has_handler_type(ConcurrentRotatingFileHandler)
-        ) and all([self._log_path, self._log_filename]):
+        if all([self._log_path, log_filename]) and ((is_error_level_file_handler is False and self._judge_logger_has_not_handler_type(HANDLER_TYPE_FILE))
+            or (is_error_level_file_handler is True and self._judge_logger_has_not_handler_type(HANDLER_TYPE_ERROR_FILE)) ):
             if not os.path.exists(self._log_path):
-                os.makedirs(self._log_path)
-            log_file = os.path.join(self._log_path, self._log_filename)
+                os.makedirs(self._log_path, exist_ok=True)
+            log_file = os.path.join(self._log_path, log_filename)
             file_handler = None
             if self._log_file_handler_type == 1:
                 if os_name == 'nt':
@@ -368,6 +428,8 @@ class LogManager(object):
                                                                                             maxBytes=self._log_file_size * 1024 * 1024,
                                                                                             backupCount=nb_log_config_default.LOG_FILE_BACKUP_COUNT,
                                                                                             encoding="utf-8")
+
+
                 elif os_name == 'posix':
                     # linux下可以使用ConcurrentRotatingFileHandler，进程安全的日志方式。
                     # 10进程各自写入10万条记录到同一个文件消耗100秒，还是比不切片写入速度降低10倍。因为每次检查切片大小和文件锁的原因。
@@ -378,8 +440,9 @@ class LogManager(object):
 
             elif self._log_file_handler_type == 4:
                 file_handler = WatchedFileHandler(log_file)
+
             elif self._log_file_handler_type == 2:
-                file_handler = ConcurrentDayRotatingFileHandler(self._log_filename, self._log_path, back_count=nb_log_config_default.LOG_FILE_BACKUP_COUNT)
+                file_handler = ConcurrentDayRotatingFileHandler(log_filename, self._log_path, back_count=nb_log_config_default.LOG_FILE_BACKUP_COUNT)
             elif self._log_file_handler_type == 3:
                 file_handler = FileHandler(log_file, mode='a', encoding='utf-8')
             elif self._log_file_handler_type == 5:
@@ -387,75 +450,117 @@ class LogManager(object):
                                                              maxBytes=self._log_file_size * 1024 * 1024,
                                                              backupCount=nb_log_config_default.LOG_FILE_BACKUP_COUNT,
                                                              encoding="utf-8")
-            file_handler.setLevel(self._logger_level2)
-            self.__add_a_hanlder(file_handler)
+            elif self._log_file_handler_type == 6:
+                file_handler = BothDayAndSizeRotatingFileHandler(file_name=log_filename, log_path=self._log_path,
+                                                                 back_count=nb_log_config_default.LOG_FILE_BACKUP_COUNT, max_bytes=self._log_file_size * 1024 * 1024)
+            elif self._log_file_handler_type == 7:
+                from nb_log.handlers_loguru import LoguruFileHandler
+                logger_name_new = self._logger_name if not is_error_level_file_handler else f'{self._logger_name}_error'
+                file_handler = LoguruFileHandler(logger_name=self._logger_name,sink=log_file)
 
-        # REMIND 添加mongo日志。
-        if not self._judge_logger_has_handler_type(MongoHandler) and self._mongo_url:
-            handler = MongoHandler(self._mongo_url)
-            handler.setLevel(self._logger_level2)
+            if is_error_level_file_handler:
+                setattr(file_handler, MANUAL_HANLDER_TYPE, HANDLER_TYPE_ERROR_FILE)
+                file_handler.setLevel(logging.ERROR)
+            else:
+                setattr(file_handler, MANUAL_HANLDER_TYPE, HANDLER_TYPE_FILE)
+                file_handler.setLevel(logging.DEBUG)
+            file_handler.setFormatter(self._formatter)
+            self.logger.addHandler(file_handler)
+
+    def __add_handlers(self):
+        pass
+
+        # REMIND 添加控制台日志
+        if self._judge_logger_has_not_handler_type(HANDLER_TYPE_STREAM) and self._is_add_stream_handler:
+            handler = ColorHandler() if not self._do_not_use_color_handler else logging.StreamHandler()  # 不使用streamhandler，使用自定义的彩色日志
+            if self._is_use_loguru_stream_handler:
+                from nb_log.handlers_loguru import LoguruStreamHandler
+                handler = LoguruStreamHandler(self._logger_name,sink=sys.stdout)
+            # handler = logging.StreamHandler()
+            handler.setLevel(self._logger_level)
+            setattr(handler,MANUAL_HANLDER_TYPE,HANDLER_TYPE_STREAM)
             self.__add_a_hanlder(handler)
 
-        if not self._judge_logger_has_handler_type(
-            ElasticHandler) and self._is_add_elastic_handler and nb_log_config_default.RUN_ENV == 'test':  # 使用kafka。不直接es。
+        self.__add_file_hanlder(self._log_filename, is_error_level_file_handler=False)
+        self.__add_file_hanlder(self._error_log_filename, is_error_level_file_handler=True)
+
+        # REMIND 添加mongo日志。
+        # if not self._judge_logger_has_handler_type(MongoHandler) and self._mongo_url:
+        if self._mongo_url :
+            from nb_log.handlers_more import MongoHandler
+            if  self._judge_logger_has_not_handler_type(MongoHandler):
+                handler = MongoHandler(self._mongo_url)
+                handler.setLevel(self._logger_level)
+                self.__add_a_hanlder(handler)
+
+        if self._is_add_elastic_handler:
             """
             生产环境使用阿里云 oss日志，不使用这个。
             """
-            handler = ElasticHandler([nb_log_config_default.ELASTIC_HOST], nb_log_config_default.ELASTIC_PORT)
-            handler.setLevel(self._logger_level2)
-            self.__add_a_hanlder(handler)
+            from nb_log.handlers_more import ElasticHandler
+            if  self._judge_logger_has_not_handler_type(ElasticHandler):
+                handler = ElasticHandler([nb_log_config_default.ELASTIC_HOST], nb_log_config_default.ELASTIC_PORT)
+                handler.setLevel(self._logger_level)
+                self.__add_a_hanlder(handler)
 
         # REMIND 添加kafka日志。
         # if self._is_add_kafka_handler:
-        if not self._judge_logger_has_handler_type(
-            KafkaHandler) and nb_log_config_default.RUN_ENV == 'test' \
-            and nb_log_config_default.ALWAYS_ADD_KAFKA_HANDLER_IN_TEST_ENVIRONENT:
-            handler = KafkaHandler(nb_log_config_default.KAFKA_BOOTSTRAP_SERVERS, )
-            handler.setLevel(self._logger_level2)
-            self.__add_a_hanlder(handler)
+        if nb_log_config_default.RUN_ENV == 'test' and nb_log_config_default.ALWAYS_ADD_KAFKA_HANDLER_IN_TEST_ENVIRONENT:
+            from nb_log.handlers_more import KafkaHandler
+            if  self._judge_logger_has_not_handler_type(KafkaHandler):
+                handler = KafkaHandler(nb_log_config_default.KAFKA_BOOTSTRAP_SERVERS, )
+                handler.setLevel(self._logger_level)
+                self.__add_a_hanlder(handler)
 
         # REMIND 添加钉钉日志。
-        if not self._judge_logger_has_handler_type(DingTalkHandler) and self._ding_talk_token:
+        if  self._judge_logger_has_not_handler_type(DingTalkHandler) and self._ding_talk_token:
             handler = DingTalkHandler(self._ding_talk_token, self._ding_talk_time_interval)
-            handler.setLevel(self._logger_level2)
+            handler.setLevel(self._logger_level)
             self.__add_a_hanlder(handler)
 
-        if not self._judge_logger_has_handler_type(CompatibleSMTPSSLHandler) and self._is_add_mail_handler:
+        if  self._judge_logger_has_not_handler_type(CompatibleSMTPSSLHandler) and self._is_add_mail_handler:
             handler = CompatibleSMTPSSLHandler(**self._mail_handler_config.get_dict())
-            handler.setLevel(self._logger_level2)
+            handler.setLevel(self._logger_level)
             self.__add_a_hanlder(handler)
 
 
 @lru_cache()  # LogManager 本身也支持无限实例化
 def get_logger(name: typing.Union[str, None], *, log_level_int: int = None, is_add_stream_handler=True,
+               is_use_loguru_stream_handler:bool=None,
                do_not_use_color_handler=None, log_path=None,
-               log_filename=None, log_file_size: int = None,
+               log_filename=None,
+               error_log_filename=None,
+               log_file_size: int = None,
                log_file_handler_type: int = None,
                mongo_url=None, is_add_elastic_handler=False, is_add_kafka_handler=False,
                ding_talk_token=None, ding_talk_time_interval=60,
                mail_handler_config: MailHandlerConfig = MailHandlerConfig(), is_add_mail_handler=False,
-               formatter_template: int = None) -> logging.Logger:
+               formatter_template: typing.Union[int, logging.Formatter] = None) -> logging.Logger:
     """
     重写一遍，是为了更好的pycharm自动补全，所以不用**kwargs的写法。
     如果太喜欢函数调用了，可以使用这种.
     get_logger_and_add_handlers是LogManager类最常用的公有方法，其他方法使用场景的频率比较低，
     但如果要使用那些低频率功能，还是要亲自调用LogManger类，而不是仅仅只了解此函数的用法。
-       :param name: 日志命名空间，重要。
-       :param log_level_int: 日志输出级别，设置为 1 2 3 4 5，分别对应原生logging.DEBUG(10)，logging.INFO(20)，
-       logging.WARNING(30)，logging.ERROR(40),logging.CRITICAL(50)级别，现在可以直接用10 20 30 40 50了，兼容了。
-
+        :param name 日志命名空间，这个是最重要最难理解的一个入参，很多pythoner到现在还不知道name是什么作用。日志命名空间，意义非常非常非常重要，有些人到现在还不知道 logging.getLogger() 第一个入参的作用，太low了。不同的name的logger可以表现出不同的行为。
+                例如让 aa命名空间的日志打印控制台并且写入到文件，并且只记录info级别以上，让 bb 命名空间的日志仅仅打印控制台，并且打印debug以上级别，
+                这种就可以通过不同的日志命名空间做到。
+        :param log_level_int: 日志输出级别，设置为 1 2 3 4 5，分别对应原生logging.DEBUG(10)，logging.INFO(20)，logging.WARNING(30)，logging.ERROR(40),logging.CRITICAL(50)级别，现在可以直接用10 20 30 40 50了，兼容了。
        :param is_add_stream_handler: 是否打印日志到控制台
+       :param is_use_loguru_stream_handler  是否使用 loguru的控制台打印，如果为None，使用 nb_log_config.py的DEFAULUT_IS_USE_LOGURU_STREAM_HANDLER 值。
        :param do_not_use_color_handler :是否禁止使用color彩色日志
        :param log_path: 设置存放日志的文件夹路径,如果不设置，则取nb_log_config.LOG_PATH，如果配置中也没指定则自动在代码所在磁盘的根目录创建/pythonlogs文件夹，
               非windwos下要注意账号权限问题(如果python没权限在根目录建/pythonlogs，则需要手动先创建好)
-       :param log_filename: 日志的名字，仅当log_path和log_filename都不为None时候才写入到日志文件。
+       :param log_filename: 日志文件名字，仅当log_path和log_filename都不为None时候才写入到日志文件。
+       :param error_log_filename :错误日志文件名字，如果文件名不为None，那么error级别以上日志自动写入到这个错误文件。
        :param log_file_size :日志大小，单位M，默认100M
-       :param log_file_handler_type :这个值可以设置为1 2 3 4 四种值，1为使用多进程安全按日志文件大小切割的文件日志，
+       :param log_file_handler_type :这个值可以设置为1 2 3 4 5 6 7，1为使用多进程安全按日志文件大小切割的文件日志
               2为多进程安全按天自动切割的文件日志，同一个文件，每天生成一个日志
               3为不自动切割的单个文件的日志(不切割文件就不会出现所谓进程安不安全的问题)
               4为 WatchedFileHandler，这个是需要在linux下才能使用，需要借助lograte外力进行日志文件的切割，多进程安全。
               5 为第三方的concurrent_log_handler.ConcurrentRotatingFileHandler按日志文件大小切割的文件日志，
                 这个是采用了文件锁，多进程安全切割，文件锁在linux上使用fcntl性能还行，win上使用win32con性能非常惨。按大小切割建议不要选第5个个filehandler而是选择第1个。
+              6 为作者发明的高性能多进程安全，同时按大小和时间切割的文件日志handler
+              7 为 loguru的 文件日志记录器
        :param mongo_url : mongodb的连接，为None时候不添加mongohandler
        :param is_add_elastic_handler: 是否记录到es中。
        :param is_add_kafka_handler: 日志是否发布到kafka。
@@ -463,7 +568,7 @@ def get_logger(name: typing.Union[str, None], *, log_level_int: int = None, is_a
        :param ding_talk_time_interval : 时间间隔，少于这个时间不发送钉钉消息
        :param mail_handler_config : 邮件配置
        :param is_add_mail_handler :是否发邮件
-        :param formatter_template :日志模板，如果为数字，则为nb_log_config.py字典formatter_dict的键对应的模板，
+       :param formatter_template :日志模板，如果为数字，则为nb_log_config.py字典formatter_dict的键对应的模板，
                                 1为formatter_dict的详细模板，2为简要模板,5为最好模板。
                                 如果为logging.Formatter对象，则直接使用用户传入的模板。
        :type log_level_int :int
@@ -489,7 +594,18 @@ def get_logger_with_filehanlder(name: str) -> logging.Logger:
     return LogManager(name).get_logger_and_add_handlers(log_filename=name + '.log')
 
 
-class LoggerMixin(object):
+class LoggerLevelSetterMixin:
+    # noinspection PyUnresolvedReferences
+    def set_log_level(self, log_level=10):
+        try:
+            self.logger.setLevel(log_level)
+        except AttributeError as e:
+            very_nb_print(e)
+
+        return self
+
+
+class LoggerMixin(LoggerLevelSetterMixin):
     """
     主要是生成把类名作为日志命名空间的logger，方便被混入类直接使用self.logger，不需要手动实例化get_logger。
     """
@@ -518,7 +634,7 @@ class LoggerMixin(object):
         logger_name_key = self.logger_full_name + '2'
         if logger_name_key not in self.subclass_logger_dict:
             logger_var = LogManager(self.logger_full_name).get_logger_and_add_handlers(
-                log_filename=self.logger_full_name + '.log', log_file_size=50)
+                log_filename=self.logger_full_name + '.log', )
             self.subclass_logger_dict[logger_name_key] = logger_var
             return logger_var
         else:
@@ -533,19 +649,23 @@ class LoggerMixinDefaultWithFileHandler(LoggerMixin):
         logger_name_key = self.logger_full_name + '3'
         if logger_name_key not in self.subclass_logger_dict:
             logger_var = LogManager(self.logger_full_name).get_logger_and_add_handlers(
-                log_filename=self.logger_full_name + '.log', log_file_size=50)
+                log_filename=self.logger_full_name + '.log', )
             self.subclass_logger_dict[logger_name_key] = logger_var
             return logger_var
         else:
             return self.subclass_logger_dict[logger_name_key]
 
 
-class LoggerLevelSetterMixin:
-    # noinspection PyUnresolvedReferences
-    def set_log_level(self, log_level=10):
-        try:
-            self.logger.setLevel(log_level)
-        except AttributeError as e:
-            very_nb_print(e)
+FileLoggerMixin = LoggerMixinDefaultWithFileHandler
 
-        return self
+
+class MetaTypeLogger(type):
+    def __init__(cls, name, bases, attrs):
+        super().__init__(name, bases, attrs)
+        cls.logger = get_logger(name)
+
+
+class MetaTypeFileLogger(type):
+    def __init__(cls, name, bases, attrs):
+        super().__init__(name, bases, attrs)
+        cls.logger = get_logger(name, log_filename=f'{name}.log')
